@@ -33,13 +33,15 @@ from gettext import gettext as _
 import cairo
 from math import pi
 import os
-from _thread import start_new_thread
-from time import sleep
+from threading import Thread
 from gnomemusic import log
 from gnomemusic.grilo import grilo
 import logging
+from queue import Queue
 logger = logging.getLogger(__name__)
 
+WORKER_THREADS = 2
+
 
 @log
 def _make_icon_frame(pixbuf, path=None):
@@ -111,13 +113,34 @@ class AlbumArtCache:
 
         return title
 
+    def worker(self, id):
+        while True:
+            try:
+                item = self.thread_queue.get()
+                item.setDaemon(True)
+                item.start()
+                item.join(30)
+                self.thread_queue.task_done()
+            except Exception as e:
+                logger.warn("worker %d item %s: error %s" % (id, item, str(e)))
+
     @log
     def __init__(self):
         try:
             self.cacheDir = os.path.join(GLib.get_user_cache_dir(), 'media-art')
-            Gio.file_new_for_path(self.cacheDir).make_directory(None)
-        except:
-            pass
+            if not os.path.exists(self.cacheDir):
+                Gio.file_new_for_path(self.cacheDir).make_directory(None)
+        except Exception as e:
+            logger.warn("Error: %s" % e)
+
+        try:
+            self.thread_queue = Queue()
+            for i in range(WORKER_THREADS):
+                t = Thread(target=self.worker, args=(i,))
+                t.setDaemon(True)
+                t.start()
+        except Exception as e:
+            logger.warn("Error: %s" % e)
 
     @log
     def get_default_icon(self, width, height):
@@ -145,9 +168,10 @@ class AlbumArtCache:
     @log
     def lookup(self, item, width, height, callback, itr, artist, album):
         try:
-            start_new_thread(self.lookup_worker, (item, width, height, callback, itr, artist, album))
+            t = Thread(target=self.lookup_worker, args=(item, width, height, callback, itr, artist, album))
+            self.thread_queue.put(t)
         except Exception as e:
-            logger.warn("Error: %s" % e)
+            logger.warn("Error: %s" % e.__class__)
 
     @log
     def lookup_worker(self, item, width, height, callback, itr, artist, album):
@@ -192,8 +216,8 @@ class AlbumArtCache:
                                              (item, width, height, path, callback, itr, artist, album))
                 return
 
-            start_new_thread(self.download_worker,
-                             (item, width, height, path, callback, itr, artist, album, uri))
+            t = Thread(target=self.download_worker, args=(item, width, height, path, callback, itr, artist, album, uri))
+            self.thread_queue.put(t)
         except Exception as e:
             logger.warn("Error: %s" % e)
 
@@ -215,7 +239,8 @@ class AlbumArtCache:
                 self.finish(item, None, None, callback, itr)
                 return
 
-            start_new_thread(self.download_worker, (item, width, height, path, callback, itr, artist, album, uri))
+            t = Thread(target=self.download_worker, args=(item, width, height, path, callback, itr, artist, album, uri))
+            self.thread_queue.put(t)
         except Exception as e:
             logger.warn("Error: %s" % e)
 
