@@ -30,8 +30,11 @@ from dbus.mainloop.glib import DBusGMainLoop
 
 from gnomemusic.player import PlaybackStatus, RepeatType
 from gnomemusic.albumArtCache import AlbumArtCache
+from gnomemusic.grilo import grilo
+from gnomemusic.playlists import Playlists
 
 from gettext import gettext as _
+from gi.repository import GLib
 from gi.repository import Grl
 from gnomemusic import log
 import logging
@@ -41,6 +44,8 @@ logger = logging.getLogger(__name__)
 class MediaPlayer2Service(dbus.service.Object):
     MEDIA_PLAYER2_IFACE = 'org.mpris.MediaPlayer2'
     MEDIA_PLAYER2_PLAYER_IFACE = 'org.mpris.MediaPlayer2.Player'
+    MEDIA_PLAYER2_TRACKLIST_IFACE = 'org.mpris.MediaPlayer2.TrackList'
+    MEDIA_PLAYER2_PLAYLISTS_IFACE = 'org.mpris.MediaPlayer2.Playlists'
 
     def __init__(self, app):
         DBusGMainLoop(set_as_default=True)
@@ -55,6 +60,15 @@ class MediaPlayer2Service(dbus.service.Object):
         self.player.connect('volume-changed', self._on_volume_changed)
         self.player.connect('prev-next-invalidated', self._on_prev_next_invalidated)
         self.player.connect('seeked', self._on_seeked)
+        self.player.connect('playlist-changed', self._on_playlist_changed)
+        playlists = Playlists.get_default()
+        playlists.connect('playlist-created', self._on_playlists_count_changed)
+        playlists.connect('playlist-deleted', self._on_playlists_count_changed)
+        grilo.connect('ready', self._on_grilo_ready)
+        self.playlists = []
+        self.playlist = None
+        self.playlist_insert_handler = 0
+        self.playlist_delete_handler = 0
         self.first_song_handler = 0
 
     @log
@@ -77,13 +91,14 @@ class MediaPlayer2Service(dbus.service.Object):
             return 'Playlist'
 
     @log
-    def _get_metadata(self):
-        media = self.player.get_current_media()
+    def _get_metadata(self, media=None):
+        if not media:
+            media = self.player.get_current_media()
         if not media:
             return {}
 
         metadata = {
-            'mpris:trackid': '/org/mpris/MediaPlayer2/Track/%s' % media.get_id(),
+            'mpris:trackid': self._get_media_id(media),
             'xesam:url': media.get_url()
         }
 
@@ -174,6 +189,70 @@ class MediaPlayer2Service(dbus.service.Object):
 
         return metadata
 
+    @log
+    def _get_media_id(self, media):
+        return '/org/mpris/MediaPlayer2/TrackList/%s' % \
+            (media.get_id() if media else 'NoTrack')
+
+    @log
+    def _get_media_from_id(self, track_id):
+        for track in self.player.playlist:
+            media = track[self.player.playlistField]
+            if track_id == self._get_media_id(media):
+                return media
+        return None
+
+    @log
+    def _get_track_list(self):
+        if self.player.playlist:
+            return [self._get_media_id(track[self.player.playlistField])
+                    for track in self.player.playlist]
+        else:
+            return []
+
+    @log
+    def _get_playlist_path(self, playlist):
+        return '/org/mpris/MediaPlayer2/Playlist/%s' % \
+            (playlist.get_id() if playlist else 'Invalid')
+
+    @log
+    def _get_playlist_from_path(self, playlist_path):
+        for playlist in self.playlists:
+            if playlist_path == self._get_playlist_path(playlist):
+                return playlist
+        return None
+
+    @log
+    def _get_playlist_from_id(self, playlist_id):
+        for playlist in self.playlists:
+            if playlist_id == playlist.get_id():
+                return playlist
+        return None
+
+    @log
+    def _get_playlists(self, callback):
+        playlists = []
+
+        def populate_callback(source, param, item, remaining=0, data=None):
+            if item:
+                playlists.append(item)
+            else:
+                callback(playlists)
+
+        if grilo.tracker:
+            GLib.idle_add(grilo.populate_playlists, 0, populate_callback)
+        else:
+            callback(playlists)
+
+    @log
+    def _get_active_playlist(self):
+        playlist = self._get_playlist_from_id(self.player.playlistId) \
+            if self.player.playlistType == 'Playlist' else None
+        playlistName = AlbumArtCache.get_media_title(playlist) \
+            if playlist else ''
+        return (playlist is not None,
+                (self._get_playlist_path(playlist), playlistName, ''))
+
     @log
     def _on_current_changed(self, player, data=None):
         if self.player.repeat == RepeatType.SONG:
@@ -243,6 +322,60 @@ class MediaPlayer2Service(dbus.service.Object):
     def _on_seeked(self, player, position, data=None):
         self.Seeked(position)
 
+    @log
+    def _on_playlist_changed(self, player, data=None):
+        if self.playlist:
+            if self.playlist_insert_handler:
+                self.playlist.disconnect(self.playlist_insert_handler)
+            if self.playlist_delete_handler:
+                self.playlist.disconnect(self.playlist_delete_handler)
+
+        self.playlist = self.player.playlist
+        self._on_playlist_modified()
+
+        self.PropertiesChanged(self.MEDIA_PLAYER2_PLAYLISTS_IFACE,
+                               {
+                                   'ActivePlaylist': self._get_active_playlist(),
+                               },
+                               [])
+
+        self.playlist_insert_handler = \
+            self.playlist.connect('row-inserted', self._on_playlist_modified)
+        self.playlist_delete_handler = \
+            self.playlist.connect('row-deleted', self._on_playlist_modified)
+
+    @log
+    def _on_playlist_modified(self, path=None, _iter=None, data=None):
+        path = self.player.currentTrack.get_path()
+        currentTrack = self.player.playlist[path][self.player.playlistField]
+        track_list = self._get_track_list()
+        self.TrackListReplaced(track_list, self._get_media_id(currentTrack))
+        self.PropertiesChanged(self.MEDIA_PLAYER2_TRACKLIST_IFACE,
+                               {
+                                   'Tracks': track_list,
+                               },
+                               [])
+
+    @log
+    def _reload_playlists(self):
+        def get_playlists_callback(playlists):
+            self.playlists = playlists
+            self.PropertiesChanged(self.MEDIA_PLAYER2_PLAYLISTS_IFACE,
+                                   {
+                                       'PlaylistCount': len(playlists),
+                                   },
+                                   [])
+
+        self._get_playlists(get_playlists_callback)
+
+    @log
+    def _on_playlists_count_changed(self, playlists, item):
+        self._reload_playlists()
+
+    @log
+    def _on_grilo_ready(self, grilo):
+        self._reload_playlists()
+
     @dbus.service.method(dbus_interface=MEDIA_PLAYER2_IFACE)
     def Raise(self):
         self.app.do_activate()
@@ -307,6 +440,80 @@ class MediaPlayer2Service(dbus.service.Object):
     def Seeked(self, position):
         pass
 
+    @dbus.service.method(dbus_interface=MEDIA_PLAYER2_TRACKLIST_IFACE,
+                         in_signature='ao', out_signature='aa{sv}')
+    def GetTracksMetadata(self, track_ids):
+        metadata = []
+        for track_id in track_ids:
+            metadata.append(self._get_metadata(self._get_media_from_id(track_id)))
+        return metadata
+
+    @dbus.service.method(dbus_interface=MEDIA_PLAYER2_TRACKLIST_IFACE,
+                         in_signature='sob')
+    def AddTrack(self, uri, after_track, set_as_current):
+        pass
+
+    @dbus.service.method(dbus_interface=MEDIA_PLAYER2_TRACKLIST_IFACE,
+                         in_signature='o')
+    def RemoveTrack(self, track_id):
+        pass
+
+    @dbus.service.method(dbus_interface=MEDIA_PLAYER2_TRACKLIST_IFACE,
+                         in_signature='o')
+    def GoTo(self, track_id):
+        for track in self.player.playlist:
+            media = track[self.player.playlistField]
+            if track_id == self._get_media_id(media):
+                self.player.set_playlist(self.player.playlistType,
+                                         self.player.playlistId,
+                                         self.player.playlist,
+                                         track.iter,
+                                         self.player.playlistField)
+                self.player.play()
+                return
+
+    @dbus.service.signal(dbus_interface=MEDIA_PLAYER2_TRACKLIST_IFACE,
+                         signature='aoo')
+    def TrackListReplaced(self, tracks, current_track):
+        pass
+
+    @dbus.service.signal(dbus_interface=MEDIA_PLAYER2_TRACKLIST_IFACE,
+                         signature='a{sv}o')
+    def TrackAdded(self, metadata, after_track):
+        pass
+
+    @dbus.service.signal(dbus_interface=MEDIA_PLAYER2_TRACKLIST_IFACE,
+                         signature='o')
+    def TrackRemoved(self, track_id):
+        pass
+
+    @dbus.service.signal(dbus_interface=MEDIA_PLAYER2_TRACKLIST_IFACE,
+                         signature='oa{sv}')
+    def TrackMetadataChanged(self, track_id, metadata):
+        pass
+
+    @dbus.service.method(dbus_interface=MEDIA_PLAYER2_PLAYLISTS_IFACE,
+                         in_signature='o')
+    def ActivatePlaylist(self, playlist_path):
+        playlist_id = self._get_playlist_from_path(playlist_path).get_id()
+        self.app._window.views[3].activate_playlist(playlist_id)
+
+    @dbus.service.method(dbus_interface=MEDIA_PLAYER2_PLAYLISTS_IFACE,
+                         in_signature='uusb', out_signature='a(oss)')
+    def GetPlaylists(self, index, max_count, order, reverse):
+        if order != 'Alphabetical':
+            return []
+        playlists = [(self._get_playlist_path(playlist),
+                      AlbumArtCache.get_media_title(playlist) or '', '')
+                     for playlist in self.playlists]
+        return playlists[index:index + max_count] if not reverse \
+            else playlists[index + max_count - 1:index - 1 if index - 1 >= 0 else None:-1]
+
+    @dbus.service.signal(dbus_interface=MEDIA_PLAYER2_PLAYLISTS_IFACE,
+                         signature='(oss)')
+    def PlaylistChanged(self, playlist):
+        pass
+
     @dbus.service.method(dbus_interface=dbus.PROPERTIES_IFACE,
                          in_signature='ss', out_signature='v')
     def Get(self, interface_name, property_name):
@@ -318,8 +525,10 @@ class MediaPlayer2Service(dbus.service.Object):
         if interface_name == self.MEDIA_PLAYER2_IFACE:
             return {
                 'CanQuit': True,
+                'Fullscreen': False,
+                'CanSetFullscreen': False,
                 'CanRaise': True,
-                'HasTrackList': False,
+                'HasTrackList': True,
                 'Identity': 'Music',
                 'DesktopEntry': 'gnome-music',
                 'SupportedUriSchemes': [
@@ -350,6 +559,17 @@ class MediaPlayer2Service(dbus.service.Object):
                 'CanSeek': True,
                 'CanControl': True,
             }
+        elif interface_name == self.MEDIA_PLAYER2_TRACKLIST_IFACE:
+            return {
+                'Tracks': self._get_track_list(),
+                'CanEditTracks': False,
+            }
+        elif interface_name == self.MEDIA_PLAYER2_PLAYLISTS_IFACE:
+            return {
+                'PlaylistCount': len(self.playlists),
+                'Orderings': ['Alphabetical'],
+                'ActivePlaylist': self._get_active_playlist(),
+            }
         else:
             raise dbus.exceptions.DBusException(
                 'org.mpris.MediaPlayer2.GnomeMusic',
@@ -360,7 +580,8 @@ class MediaPlayer2Service(dbus.service.Object):
                          in_signature='ssv')
     def Set(self, interface_name, property_name, new_value):
         if interface_name == self.MEDIA_PLAYER2_IFACE:
-            pass
+            if property_name == 'Fullscreen':
+                pass
         elif interface_name == self.MEDIA_PLAYER2_PLAYER_IFACE:
             if property_name == 'Rate':
                 pass
