@@ -65,6 +65,7 @@ class Player(GObject.GObject):
 
     __gsignals__ = {
         'playing-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
+        'playlist-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
         'playlist-item-changed': (GObject.SIGNAL_RUN_FIRST, None, (Gtk.TreeModel, Gtk.TreeIter)),
         'current-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
         'playback-status-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
@@ -117,7 +118,7 @@ class Player(GObject.GObject):
             return
 
         if not url.startswith("file://"):
-            logger.debug("Skipping discovery of %s as not a local file" % url)
+            logger.debug("Skipping discovery of %s as a remote url" % url)
             return
 
         obj = (callback, data)
@@ -510,20 +511,23 @@ class Player(GObject.GObject):
     def set_playlist(self, type, id, model, iter, field):
         self.stop()
 
-        if self.playlist_insert_handler:
-            self.playlist.disconnect(self.playlist_insert_handler)
-        if self.playlist_delete_handler:
-            self.playlist.disconnect(self.playlist_delete_handler)
+        old_playlist = self.playlist
+        if old_playlist != model:
+            self.playlist = model
+            if self.playlist_insert_handler:
+                old_playlist.disconnect(self.playlist_insert_handler)
+            if self.playlist_delete_handler:
+                old_playlist.disconnect(self.playlist_delete_handler)
 
-        self.playlist = model
         self.playlistType = type
         self.playlistId = id
         self.currentTrack = Gtk.TreeRowReference.new(model, model.get_path(iter))
         self.playlistField = field
 
-        self.playlist_insert_handler = model.connect('row-inserted', self._on_playlist_size_changed)
-        self.playlist_delete_handler = model.connect('row-deleted', self._on_playlist_size_changed)
-
+        if old_playlist != model:
+            self.playlist_insert_handler = model.connect('row-inserted', self._on_playlist_size_changed)
+            self.playlist_delete_handler = model.connect('row-deleted', self._on_playlist_size_changed)
+            self.emit('playlist-changed')
         self.emit('current-changed')
 
     @log
@@ -553,11 +557,11 @@ class Player(GObject.GObject):
         self.repeatBtnImage = self._ui.get_object('playlistRepeat')
 
         if Gtk.Settings.get_default().get_property('gtk_application_prefer_dark_theme'):
-            color = Gdk.Color(red=65535, green=65535, blue=65535)
+            color = Gdk.RGBA(red=1.0, green=1.0, blue=1.0, alpha=1.0)
         else:
-            color = Gdk.Color(red=0, green=0, blue=0)
-        self._playImage.modify_fg(Gtk.StateType.ACTIVE, color)
-        self._pauseImage.modify_fg(Gtk.StateType.ACTIVE, color)
+            color = Gdk.RGBA(red=0.0, green=0.0, blue=0.0, alpha=0.0)
+        self._playImage.override_color(Gtk.StateFlags.ACTIVE, color)
+        self._pauseImage.override_color(Gtk.StateFlags.ACTIVE, color)
 
         self._sync_repeat_image()
 
