@@ -25,22 +25,45 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
+from gettext import gettext as _
 from gi.repository import GLib, Tracker
+import os
+import logging
+logger = logging.getLogger(__name__)
 
 
 class Query():
-    music_folder = None
-    MUSIC_DIR = None
-    download_folder = None
-    DOWNLOAD_DIR = None
-    try:
-        music_folder = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_MUSIC)
-        MUSIC_DIR = Tracker.sparql_escape_string(GLib.filename_to_uri(music_folder))
-        download_folder = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DOWNLOAD)
-        DOWNLOAD_DIR = Tracker.sparql_escape_string(GLib.filename_to_uri(download_folder))
+    music_folder = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_MUSIC)
+    MUSIC_URI = Tracker.sparql_escape_string(GLib.filename_to_uri(music_folder))
+    download_folder = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DOWNLOAD)
+    DOWNLOAD_URI = Tracker.sparql_escape_string(GLib.filename_to_uri(download_folder))
 
-    except TypeError:
-        print("XDG user dirs are not set")
+    for folder in [music_folder, download_folder]:
+        if os.path.islink(folder):
+            logger.warn("%s is a symlink, this folder will be omitted" % folder)
+        else:
+            i = len(next(os.walk(folder))[2])
+            logger.debug("Found %d files in %s" % (i, folder))
+
+
+    @staticmethod
+    def order_by_statement(attr):
+        """Returns a SPARQL ORDER BY statement sorting by the given attribute, ignoring
+            articles as defined in _("the"). 'Attr' should be given without parentheses,
+            e.g., "attr='?author'"."""
+        return_statement = "fn:lower-case(%(attribute)s)" % {'attribute' : attr }
+        # TRANSLATORS: _("the") should be a space-separated list of all-lowercase articles
+            # (such as 'the') that should be ignored when alphabetizing artists/albums. This
+            # list should include 'the' regardless of language. If some articles occur more
+            # frequently than others, most common should appear first, least common last.
+        for article in reversed(_("the").split(" ")):
+            return_statement = '''IF(fn:starts-with(fn:lower-case(%(attribute)s), "%(article)s"),
+            fn:substring(fn:lower-case(%(attribute)s), %(substr_start)s),
+            %(nested_if)s)''' % {'attribute' : attr,
+                'article' : article+" ",
+                'substr_start' : str(len(article)+2),
+                'nested_if' : return_statement }
+        return return_statement
 
     @staticmethod
     def all_albums():
@@ -84,8 +107,8 @@ class Query():
         )
     }
     '''.replace('\n', ' ').strip() % {
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI
         }
 
         return query
@@ -214,11 +237,15 @@ class Query():
                 }
             )
         }
-    ORDER BY fn:lower-case(?title) ?author ?albumyear
+    ORDER BY %(album_order)s
+        %(artist_order)s
+        ?albumyear
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI,
+            'album_order' : Query.order_by_statement("?title"),
+            'artist_order' : Query.order_by_statement("?author")
         }
 
         return query
@@ -363,11 +390,15 @@ class Query():
                 }
             )
         }
-    ORDER BY fn:lower-case(?author) ?albumyear nie:title(?album)
+    ORDER BY %(artist_order)s
+        ?albumyear
+        %(album_order)s
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI,
+            'artist_order': Query.order_by_statement("?author"),
+            'album_order' : Query.order_by_statement("nie:title(?album)")
         }
 
         return query
@@ -405,8 +436,8 @@ class Query():
     ORDER BY tracker:added(?song)
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI
         }
 
         return query
@@ -438,11 +469,11 @@ class Query():
                 )
             }
         }
-    ORDER BY fn:lower-case(?title) ?author ?albumyear
+    ORDER BY fn:lower-case(?title)
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI
         }
 
         return query
@@ -488,8 +519,8 @@ class Query():
          tracker:added(?song)
     '''.replace('\n', ' ').strip() % {
             'album_id': album_id,
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI
         }
 
         return query
@@ -532,8 +563,8 @@ class Query():
             'playlist_id': playlist_id,
             'filter_clause':
                 filter_clause or 'tracker:id(?playlist) = ' + playlist_id,
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI
         }
 
         return query
@@ -562,8 +593,8 @@ class Query():
     }
     """.replace("\n", " ").strip() % {
             'album_id': album_id,
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI
         }
         return query
 
@@ -608,8 +639,8 @@ class Query():
     }
     """.replace("\n", " ").strip() % {
             'song_id': song_id,
-            'music_dir': Query.MUSIC_DIR,
-            'download_dir': Query.DOWNLOAD_DIR
+            'music_dir': Query.MUSIC_URI,
+            'download_dir': Query.DOWNLOAD_URI
         }
         return query
 
