@@ -270,11 +270,8 @@ class ViewContainer(Gtk.Stack):
     @log
     def _on_discovered(self, info, error, _iter):
         if error:
-            try:
-                logger.warn("File will be skipped: %s\n%s" % (error.message, info.get_uri()))
-                self._model.set(_iter, [8, 10], [self.errorIconName, True])
-            except Exception:
-                pass
+            print("Info %s: error: %s" % (info, error))
+            self._model.set(_iter, [8, 10], [self.errorIconName, True])
 
     @log
     def _add_item(self, source, param, item, remaining=0, data=None):
@@ -294,7 +291,7 @@ class ViewContainer(Gtk.Stack):
                 try:
                     self.player.discoverer.discover_uri(item.get_url())
                 except:
-                    logger.warn('failed to discover url ' + item.get_url())
+                    print('failed to discover url ' + item.get_url())
                     icon_name = self.errorIconName
             self._model.set(_iter,
                             [0, 1, 2, 3, 4, 5, 7, 8, 9, 10],
@@ -334,10 +331,7 @@ class Empty(Gtk.Stack):
                            transition_type=Gtk.StackTransitionType.CROSSFADE)
         builder = Gtk.Builder()
         builder.add_from_resource('/org/gnome/Music/NoMusic.ui')
-        music_folder_path = GLib.get_user_special_dir(GLib.USER_DIRECTORY_MUSIC)
         widget = builder.get_object('container')
-        label = builder.get_object('label1')
-        label.set_label(_("No Music found!\n Put some files into the folder %s") % music_folder_path)
         self.add(widget)
         self.show_all()
 
@@ -372,10 +366,7 @@ class Albums(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        try:
-            _iter = self._model.get_iter(path)
-        except TypeError:
-            return
+        _iter = self._model.get_iter(path)
         title = self._model.get_value(_iter, 2)
         artist = self._model.get_value(_iter, 3)
         item = self._model.get_value(_iter, 5)
@@ -460,10 +451,7 @@ class Songs(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        try:
-            _iter = self._model.get_iter(path)
-        except TypeError:
-            return
+        _iter = self._model.get_iter(path)
         if self._model.get_value(_iter, 8) != self.errorIconName:
             self.player.set_playlist('Songs', None, self._model, _iter, 5)
             self.player.set_playing(True)
@@ -673,10 +661,7 @@ class Artists (ViewContainer):
 
     @log
     def _on_item_activated(self, widget, item_id, path):
-        try:
-            _iter = self._model.get_iter(path)
-        except TypeError:
-            return
+        _iter = self._model.get_iter(path)
         self._last_selection = _iter
         artist = self._model.get_value(_iter, 2)
         albums = self._artists[artist.casefold()]['albums']
@@ -799,6 +784,10 @@ class Artists (ViewContainer):
 
 
 class Playlist(ViewContainer):
+    __gsignals__ = {
+        'playlists-loaded': (GObject.SIGNAL_RUN_FIRST, None, ()),
+        'playlist-songs-loaded': (GObject.SIGNAL_RUN_FIRST, None, ()),
+    }
 
     @log
     def __init__(self, window, player):
@@ -997,6 +986,7 @@ class Playlist(ViewContainer):
     @log
     def _add_playlist_item_to_model(self, item):
         if not item:
+            self.emit('playlists-loaded')
             return
         _iter = self.playlists_model.insert_with_valuesv(
             -1,
@@ -1011,10 +1001,7 @@ class Playlist(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        try:
-            _iter = self._model.get_iter(path)
-        except TypeError:
-            return
+        _iter = self._model.get_iter(path)
         if self._model.get_value(_iter, 8) != self.errorIconName:
             self.player.set_playlist(
                 'Playlist', self.current_playlist.get_id(),
@@ -1022,6 +1009,47 @@ class Playlist(ViewContainer):
             )
             self.player.set_playing(True)
 
+    @log
+    def activate_playlist(self, playlist_id):
+        def find_and_activate_playlist():
+            for playlist in self.playlists_model:
+                if playlist[5].get_id() == playlist_id:
+                    selection = self.playlists_sidebar.get_generic_view().get_selection()
+                    if selection.iter_is_selected(playlist.iter):
+                        self._on_play_activate(None)
+                    else:
+                        selection.select_iter(playlist.iter)
+                        handler = 0
+
+                        def songs_loaded_callback(view):
+                            self.disconnect(handler)
+                            self._on_play_activate(None)
+
+                        handler = self.connect('playlist-songs-loaded', songs_loaded_callback)
+                        self.playlists_sidebar.emit('item-activated', '0', playlist.path)
+
+                    return
+
+        if self._init:
+            find_and_activate_playlist()
+        else:
+            handler = 0
+
+            def playlists_loaded_callback(view):
+                self.disconnect(handler)
+                def_handler = 0
+
+                def songs_loaded_callback(view):
+                    self.disconnect(def_handler)
+                    find_and_activate_playlist()
+
+                # Skip load of default playlist
+                def_handler = self.connect('playlist-songs-loaded', songs_loaded_callback)
+
+            handler = self.connect('playlists-loaded', playlists_loaded_callback)
+
+            self._populate()
+
     @log
     def _on_playlist_activated(self, widget, item_id, path):
         _iter = self.playlists_model.get_iter(path)
@@ -1045,6 +1073,7 @@ class Playlist(ViewContainer):
             self.view.set_model(self._model)
             self.songs_count = self._model.iter_n_children(None)
             self._update_songs_count()
+            self.emit('playlist-songs-loaded')
         else:
             self._model = Gtk.ListStore(
                 GObject.TYPE_STRING,
@@ -1071,6 +1100,7 @@ class Playlist(ViewContainer):
     @log
     def _add_item_to_model(self, item, model):
         if not item:
+            self.emit('playlist-songs-loaded')
             return
         self._offset += 1
         title = albumArtCache.get_media_title(item)
@@ -1250,10 +1280,7 @@ class Search(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        try:
-            child_path = self.filter_model.convert_path_to_child_path(path)
-        except TypeError:
-            return
+        child_path = self.filter_model.convert_path_to_child_path(path)
         _iter = self._model.get_iter(child_path)
         if self._model[_iter][11] == 'album':
             title = self._model.get_value(_iter, 2)
