@@ -31,7 +31,7 @@
 # delete this exception statement from your version.
 
 
-from gi.repository import Gtk, Gdk, Gd, GLib, GObject, Pango
+from gi.repository import Gtk, Gdk, Gd, GLib, GObject, Pango, Gio
 from gi.repository import GdkPixbuf, Grl
 from gi.repository import Tracker
 from gettext import gettext as _, ngettext
@@ -64,7 +64,7 @@ class AlbumWidget(Gtk.EventBox):
 
     @log
     def __init__(self, player):
-        super(Gtk.EventBox, self).__init__()
+        Gtk.EventBox.__init__(self)
         self.player = player
         self.iterToClean = None
 
@@ -85,7 +85,7 @@ class AlbumWidget(Gtk.EventBox):
         child_view = self.view.get_children()[0]
         child_view.set_margin_top(64)
         child_view.set_margin_bottom(64)
-        child_view.set_margin_right(32)
+        child_view.set_margin_end(32)
         self.view.remove(child_view)
         view_box.add(child_view)
         self.add(self.ui.get_object('AlbumWidget'))
@@ -309,12 +309,12 @@ class AlbumWidget(Gtk.EventBox):
         return False
 
 
-class ArtistAlbums(Gtk.VBox):
+class ArtistAlbums(Gtk.Box):
 
     @log
     def __init__(self, artist, albums, player,
                  header_bar, selection_toolbar, selectionModeAllowed=False):
-        Gtk.VBox.__init__(self)
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL)
         self.player = player
         self.artist = artist
         self.albums = albums
@@ -467,13 +467,13 @@ class AllArtistsAlbums(ArtistAlbums):
             self.add_album(item)
 
 
-class ArtistAlbumWidget(Gtk.HBox):
+class ArtistAlbumWidget(Gtk.Box):
 
     pixbuf = AlbumArtCache.get_default().get_default_icon(128, 128)
 
     @log
     def __init__(self, artist, album, player, model, header_bar, selectionModeAllowed):
-        super(Gtk.HBox, self).__init__()
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         self.player = player
         self.album = album
         self.artist = artist
@@ -502,6 +502,13 @@ class ArtistAlbumWidget(Gtk.HBox):
         self.pack_start(self.ui.get_object('ArtistAlbumWidget'), True, True, 0)
         self.show_all()
 
+        try:
+            self.settings = Gio.Settings.new('org.gnome.Music')
+            self.max_title_width = self.settings.get_int('max-width-chars')
+        except Exception as e:
+            self.max_title_width = 20
+            logger.error("Error on setting widget max-width-chars: %s" % str(e))
+
     @log
     def _on_discovered(self, info, error, song_widget):
         if error:
@@ -528,6 +535,8 @@ class ArtistAlbumWidget(Gtk.HBox):
                 title = AlbumArtCache.get_media_title(track)
                 ui.get_object('title').set_text(title)
                 ui.get_object('title').set_alignment(0.0, 0.5)
+                ui.get_object('title').set_max_width_chars(self.max_title_width)
+
                 self.songsGrid.attach(
                     song_widget,
                     int(i / (len(self.tracks) / 2)),
@@ -734,8 +743,7 @@ class PlaylistDialog():
     @log
     def _on_playlist_created(self, playlists, item):
         new_iter = self._add_item_to_model(item)
-        if self.view.get_columns():
-            self.view.set_cursor(self.model.get_path(new_iter),
-                                 self.view.get_columns()[0], False)
-            self.view.row_activated(self.model.get_path(new_iter),
-                                    self.view.get_columns()[0])
+        self.view.set_cursor(self.model.get_path(new_iter),
+                             self.view.get_columns()[0], False)
+        self.view.row_activated(self.model.get_path(new_iter),
+                                self.view.get_columns()[0])
