@@ -177,7 +177,7 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _setup_view(self):
-        self._box = Gtk.VBox()
+        self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.player = Player()
         self.selection_toolbar = SelectionToolbar()
         self.toolbar = Toolbar()
@@ -195,27 +195,22 @@ class Window(Gtk.ApplicationWindow):
         self._box.pack_start(self.player.actionbar, False, False, 0)
         self._box.pack_start(self.selection_toolbar.actionbar, False, False, 0)
         self.add(self._box)
-        count = 0
+        count = 1
         cursor = None
+        try:
+            cursor = tracker.query(Query.all_songs_count(), None)
+        except Exception as e:
+            logger.error("Tracker query crashed: %s" % e)
+            count = 0
 
-        if Query.music_folder and Query.download_folder:
-            try:
-                cursor = tracker.query(Query.all_songs_count(), None)
-                if cursor is not None and cursor.next(None):
-                    count = cursor.get_integer(0)
-            except Exception as e:
-                logger.error("Tracker query crashed: %s" % e)
-                count = 0
-
-            if count > 0:
-                self._switch_to_player_view()
-            # To revert to the No Music View when no songs are found
-            else:
-                if self.toolbar._selectionMode is False:
-                    self._switch_to_empty_view()
+        if cursor is not None and cursor.next(None):
+            count = cursor.get_integer(0)
+        if count > 0:
+            self._switch_to_player_view()
+        # To revert to the No Music View when no songs are found
         else:
-            # Revert to No Music view if XDG dirs are not set
-            self._switch_to_empty_view()
+            if self.toolbar._selectionMode is False:
+                self._switch_to_empty_view()
 
         self.toolbar._search_button.connect('toggled', self._on_search_toggled)
         self.toolbar.connect('selection-mode-changed', self._on_selection_mode_changed)
@@ -315,6 +310,12 @@ class Window(Gtk.ApplicationWindow):
             if (event.keyval == Gdk.KEY_f and
                     event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
                 self.toolbar.searchbar.toggle_bar()
+            # Go back from Album view on Alt + Left
+            if (event.keyval == Gdk.KEY_Left and
+                    event_and_modifiers == Gdk.ModifierType.MOD1_MASK):
+                if (self.toolbar._state != ToolbarState.MAIN):
+                    self.curr_view.set_visible_child(self.curr_view._grid)
+                    self.toolbar.set_state(ToolbarState.MAIN)
         else:
             # Close search bar after Esc is pressed
             if event.keyval == Gdk.KEY_Escape:
